from __future__ import annotations

from datetime import datetime  # noqa: TC003
from functools import cached_property
from typing import Literal

from pydantic import BaseModel, ConfigDict, Field

from ..schemas.fragments import Fragment, TypeFragText


class BaseForumDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    fid: int
    fname: str


class BaseUserDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    user_id: int
    portrait: str
    user_name: str
    nick_name_new: str

    @property
    def nick_name(self) -> str:
        return self.nick_name_new

    @property
    def show_name(self) -> str:
        return self.nick_name_new or self.user_name


class ThreadUserDTO(BaseUserDTO):
    model_config = ConfigDict(from_attributes=True)

    level: int
    glevel: int

    gender: Literal["UNKNOWN", "MALE", "FEMALE"]
    icons: list[str]

    is_bawu: bool
    is_vip: bool
    is_god: bool

    priv_like: Literal["PUBLIC", "FRIEND", "HIDE"]
    priv_reply: Literal["ALL", "FANS", "FOLLOW"]


class PostUserDTO(BaseUserDTO):
    model_config = ConfigDict(from_attributes=True)

    level: int
    glevel: int

    gender: Literal["UNKNOWN", "MALE", "FEMALE"]
    ip: str
    icons: list[str]

    is_bawu: bool
    is_vip: bool
    is_god: bool

    priv_like: Literal["PUBLIC", "FRIEND", "HIDE"]
    priv_reply: Literal["ALL", "FANS", "FOLLOW"]


class CommentUserDTO(BaseUserDTO):
    model_config = ConfigDict(from_attributes=True)

    level: int

    gender: Literal["UNKNOWN", "MALE", "FEMALE"]
    icons: list[str]

    is_bawu: bool
    is_vip: bool
    is_god: bool

    priv_like: Literal["PUBLIC", "FRIEND", "HIDE"]
    priv_reply: Literal["ALL", "FANS", "FOLLOW"]


class UserInfoDTO(BaseUserDTO):
    model_config = ConfigDict(from_attributes=True)

    nick_name_old: str
    tieba_uid: int

    glevel: int
    gender: Literal["UNKNOWN", "MALE", "FEMALE"]
    age: float
    post_num: int
    agree_num: int
    fan_num: int
    follow_num: int
    forum_num: int
    sign: str
    ip: str
    icons: list[str]

    is_vip: bool
    is_god: bool
    is_blocked: bool

    priv_like: Literal["PUBLIC", "FRIEND", "HIDE"]
    priv_reply: Literal["ALL", "FANS", "FOLLOW"]


class ShareThreadDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    pid: int
    tid: int
    fid: int
    fname: str

    author_id: int

    title: str
    contents: list[Fragment] = Field(default_factory=list)


class ThreadDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    pid: int
    tid: int
    fid: int
    fname: str

    author_id: int
    author: ThreadUserDTO

    title: str
    contents: list[Fragment] = Field(default_factory=list)

    is_good: bool
    is_top: bool
    is_share: bool
    is_hide: bool
    is_livepost: bool
    is_help: bool

    agree_num: int
    disagree_num: int
    reply_num: int
    view_num: int
    share_num: int
    create_time: datetime
    last_time: datetime

    thread_type: int
    tab_id: int
    share_origin: ShareThreadDTO

    @cached_property
    def text(self) -> str:
        text = "".join(frag.text for frag in self.contents if isinstance(frag, TypeFragText))
        return text


class PostDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    pid: int
    tid: int
    fid: int
    fname: str

    author_id: int
    author: PostUserDTO

    contents: list[Fragment] = Field(default_factory=list)
    sign: str
    comments: list[CommentDTO] = Field(default_factory=list)

    is_aimeme: bool
    is_thread_author: bool

    agree_num: int
    disagree_num: int
    reply_num: int
    create_time: datetime

    floor: int

    @cached_property
    def text(self) -> str:
        text = "".join(frag.text for frag in self.contents if isinstance(frag, TypeFragText))
        return text


class CommentDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    cid: int
    pid: int
    tid: int
    fid: int
    fname: str

    author_id: int
    author: CommentUserDTO

    contents: list[Fragment] = Field(default_factory=list)
    reply_to_id: int

    is_thread_author: bool

    agree_num: int
    disagree_num: int
    create_time: datetime

    floor: int

    @cached_property
    def text(self) -> str:
        text = "".join(frag.text for frag in self.contents if isinstance(frag, TypeFragText))
        return text


class PageInfoDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    page_size: int = 0
    current_page: int = 0
    total_page: int = 0
    total_count: int = 0

    has_more: bool = False
    has_prev: bool = False


class ThreadsDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    objs: list[ThreadDTO] = Field(default_factory=list)
    page: PageInfoDTO
    forum: BaseForumDTO


class PostsDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    objs: list[PostDTO] = Field(default_factory=list)
    page: PageInfoDTO
    forum: BaseForumDTO


class CommentsDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)

    objs: list[CommentDTO] = Field(default_factory=list)
    page: PageInfoDTO
    forum: BaseForumDTO



from typing import Any, cast

from tiebameow.schemas.fragments import (
    FragAtModel,
    FragEmojiModel,
    FragImageModel,
    FragItemModel,
    FragLinkModel,
    FragTextModel,
    FragUnknownModel,
)


def test_frag_text_model() -> None:
    model = FragTextModel(text="hello")
    assert model.type == "text"
    assert model.text == "hello"


def test_frag_at_model() -> None:
    model = FragAtModel(text="@user", user_id=123)
    assert model.type == "at"
    assert model.text == "@user"
    assert model.user_id == 123


def test_frag_image_model() -> None:
    model = FragImageModel(
        src="http://src",
        big_src="http://big",
        origin_src="http://origin",
        origin_size=100,
        show_width=100,
        show_height=100,
        hash="hash",
    )
    assert model.type == "image"
    assert model.src == "http://src"


def test_frag_link_model() -> None:
    model = FragLinkModel(text="http://link", title="title", raw_url="http://raw")
    assert model.type == "link"
    assert model.text == "http://link"

    # Test validator
    model_none = FragLinkModel(text="http://link", title="title", raw_url=cast("Any", None))
    assert model_none.raw_url == ""


def test_frag_emoji_model() -> None:
    model = FragEmojiModel(id="1", desc="smile")
    assert model.type == "emoji"
    assert model.id == "1"
    assert model.desc == "smile"


def test_frag_item_model() -> None:
    model = FragItemModel(text="item")
    assert model.type == "item"
    assert model.text == "item"


def test_frag_unknown_model() -> None:
    model = FragUnknownModel(raw_data="some data")
    assert model.type == "unknown"
    assert model.raw_data == "some data"


def test_fragment_union() -> None:
    # Test that Fragment union works correctly with TypeAdapter or direct instantiation if applicable
    # Since Fragment is a Union type alias, we can't instantiate it directly,
    # but we can check if models satisfy the type.

    t = FragTextModel(text="t")
    assert isinstance(t, FragTextModel)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal, overload

from ..models.dto import CommentDTO, PostDTO, ThreadDTO

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = [
    "serialize",
    "deserialize",
    "deserialize_thread",
    "deserialize_post",
    "deserialize_comment",
]


def serialize(obj: Any) -> Any:
    """将对象序列化为 JSON 可用的字典或列表。"""
    if hasattr(obj, "model_dump"):
        return obj.model_dump(mode="json")
    return obj


def _normalize_contents(data: Any) -> list[Any]:
    if isinstance(data, list):
        return data
    if isinstance(data, dict):
        return data.get("objs", []) or []
    return []


def _normalize_user(data: dict[str, Any]) -> None:
    if "author" not in data and "user" in data:
        data["author"] = data["user"]


def _preprocess_thread(data: Mapping[str, Any]) -> dict[str, Any]:
    d = dict(data)
    if "contents" in d:
        d["contents"] = _normalize_contents(d["contents"])

    _normalize_user(d)

    if "share_origin" in d and isinstance(d["share_origin"], dict):
        so = dict(d["share_origin"])
        if "contents" in so:
            so["contents"] = _normalize_contents(so["contents"])
        d["share_origin"] = so

    return d


def _preprocess_post(data: Mapping[str, Any]) -> dict[str, Any]:
    d = dict(data)
    if "contents" in d:
        d["contents"] = _normalize_contents(d["contents"])

    _normalize_user(d)

    if "comments" in d and isinstance(d["comments"], list):
        d["comments"] = [_preprocess_comment(c) if isinstance(c, dict) else c for c in d["comments"]]

    return d


def _preprocess_comment(data: Mapping[str, Any]) -> dict[str, Any]:
    d = dict(data)
    if "contents" in d:
        d["contents"] = _normalize_contents(d["contents"])

    _normalize_user(d)

    return d


def deserialize_thread(data: Mapping[str, Any]) -> ThreadDTO:
    """将 JSON/dict 反序列化为 ThreadDTO 对象。"""
    return ThreadDTO.model_validate(_preprocess_thread(data))


def deserialize_post(data: Mapping[str, Any]) -> PostDTO:
    """将 JSON/dict 反序列化为 PostDTO 对象。"""
    return PostDTO.model_validate(_preprocess_post(data))


def deserialize_comment(data: Mapping[str, Any]) -> CommentDTO:
    """将 JSON/dict 反序列化为 CommentDTO 对象。"""
    return CommentDTO.model_validate(_preprocess_comment(data))


@overload
def deserialize(item_type: Literal["thread"], data: Mapping[str, Any]) -> ThreadDTO: ...


@overload
def deserialize(item_type: Literal["post"], data: Mapping[str, Any]) -> PostDTO: ...


@overload
def deserialize(item_type: Literal["comment"], data: Mapping[str, Any]) -> CommentDTO: ...


def deserialize(
    item_type: Literal["thread", "post", "comment"], data: Mapping[str, Any]
) -> ThreadDTO | PostDTO | CommentDTO:
    """根据类型进行通用反序列化。"""
    if item_type == "thread":
        return deserialize_thread(data)
    if item_type == "post":
        return deserialize_post(data)
    if item_type == "comment":
        return deserialize_comment(data)
    raise ValueError(f"Unsupported item_type: {item_type}")



import dataclasses
from typing import Any

import pytest


# Mock aiotieba fragment classes as dataclasses because parser uses dataclasses.asdict
# Naming them to match aiotieba classes so parser can map them correctly
@dataclasses.dataclass
class FragText:
    text: str


@dataclasses.dataclass
class FragImage:
    src: str
    big_src: str
    origin_src: str
    origin_size: int
    show_width: int
    show_height: int
    hash: str


@dataclasses.dataclass
class FragAt:
    text: str
    user_id: int


@dataclasses.dataclass
class FragLink:
    text: str
    title: str
    raw_url: str


@dataclasses.dataclass
class FragEmoji:
    id: str
    desc: str


@dataclasses.dataclass
class FragItem:
    text: str


@dataclasses.dataclass
class FragUnknown:
    pass


# Mock aiotieba user and thread classes
class MockUser:
    def __init__(self, **kwargs: Any) -> None:
        for k, v in kwargs.items():
            setattr(self, k, v)


class MockThread:
    def __init__(self, **kwargs: Any) -> None:
        for k, v in kwargs.items():
            setattr(self, k, v)


@pytest.fixture
def mock_aiotieba_fragments() -> dict[str, Any]:
    return {
        "text": FragText(text="hello"),
        "image": FragImage(
            src="http://src",
            big_src="http://big",
            origin_src="http://origin",
            origin_size=100,
            show_width=100,
            show_height=100,
            hash="hash",
        ),
        "at": FragAt(text="@user", user_id=123),
        "link": FragLink(text="http://link", title="title", raw_url="http://raw"),
        "emoji": FragEmoji(id="1", desc="smile"),
        "item": FragItem(text="item"),
        "unknown": FragUnknown(),
    }



from __future__ import annotations

import dataclasses
from datetime import datetime
from typing import TYPE_CHECKING, Any, cast, overload

from ..models.dto import (
    BaseForumDTO,
    BaseUserDTO,
    CommentDTO,
    CommentsDTO,
    CommentUserDTO,
    PageInfoDTO,
    PostDTO,
    PostsDTO,
    PostUserDTO,
    ShareThreadDTO,
    ThreadDTO,
    ThreadsDTO,
    ThreadUserDTO,
    UserInfoDTO,
)
from ..schemas.fragments import FRAG_MAP, Fragment, FragUnknownModel
from ..utils.time_utils import SHANGHAI_TZ

if TYPE_CHECKING:
    from aiotieba.api._classdef.contents import (
        FragAt,
        FragEmoji,
        FragImage,
        FragItem,
        FragLink,
        FragText,
        FragTiebaPlus,
        FragUnknown,
        FragVideo,
    )
    from aiotieba.api.get_comments._classdef import Forum_c, Page_c, UserInfo_c
    from aiotieba.api.get_posts._classdef import Comment_p, Forum_p, Page_p, UserInfo_p
    from aiotieba.api.get_threads._classdef import Forum_t, Page_t, ShareThread, UserInfo_t
    from aiotieba.api.tieba_uid2user_info import UserInfo_TUid
    from aiotieba.typing import Comment, Comments, Post, Posts, Thread, Threads, UserInfo

    type AiotiebaType = Thread | Post | Comment
    type AiotiebaFragType = (
        FragAt | FragEmoji | FragImage | FragItem | FragLink | FragText | FragTiebaPlus | FragUnknown | FragVideo
    )
    type AiotiebaUserType = UserInfo_t | UserInfo_p | UserInfo_c | UserInfo_TUid
    type AiotiebaPageType = Page_t | Page_p | Page_c
    type AiotiebaForumType = Forum_t | Forum_p | Forum_c


def convert_aiotieba_fragment(obj: AiotiebaFragType | Any) -> Fragment:
    source_type_name = type(obj).__name__
    target_model_name = source_type_name.rsplit("_", 1)[0]

    target_model = FRAG_MAP.get(target_model_name)

    if target_model is None:
        return FragUnknownModel(raw_data=repr(obj))

    data_dict = dataclasses.asdict(obj)
    return target_model(**data_dict)


def convert_aiotieba_content_list(contents: list[AiotiebaFragType | Any]) -> list[Fragment]:
    if not contents:
        return []
    return [convert_aiotieba_fragment(frag) for frag in contents]


def convert_aiotieba_tiebauiduser(user: UserInfo_TUid) -> BaseUserDTO:
    return BaseUserDTO(
        user_id=user.user_id,
        portrait=user.portrait,
        user_name=user.user_name,
        nick_name_new=user.nick_name,
    )


def convert_aiotieba_threaduser(user: UserInfo_t) -> ThreadUserDTO:
    return ThreadUserDTO(
        user_id=user.user_id,
        portrait=user.portrait,
        user_name=user.user_name,
        nick_name_new=user.nick_name,
        level=user.level,
        glevel=user.glevel,
        gender=user.gender.name,
        icons=user.icons,
        is_bawu=user.is_bawu,
        is_vip=user.is_vip,
        is_god=user.is_god,
        priv_like=user.priv_like.name,
        priv_reply=user.priv_reply.name,
    )


def convert_aiotieba_postuser(user: UserInfo_p) -> PostUserDTO:
    return PostUserDTO(
        user_id=user.user_id,
        portrait=user.portrait,
        user_name=user.user_name,
        nick_name_new=user.nick_name,
        level=user.level,
        glevel=user.glevel,
        gender=user.gender.name,
        ip=user.ip,
        icons=user.icons,
        is_bawu=user.is_bawu,
        is_vip=user.is_vip,
        is_god=user.is_god,
        priv_like=user.priv_like.name,
        priv_reply=user.priv_reply.name,
    )


def convert_aiotieba_commentuser(user: UserInfo_c | UserInfo_p) -> CommentUserDTO:
    return CommentUserDTO(
        user_id=user.user_id,
        portrait=user.portrait,
        user_name=user.user_name,
        nick_name_new=user.nick_name,
        level=user.level,
        gender=user.gender.name,
        icons=user.icons,
        is_bawu=user.is_bawu,
        is_vip=user.is_vip,
        is_god=user.is_god,
        priv_like=user.priv_like.name,
        priv_reply=user.priv_reply.name,
    )


def convert_aiotieba_userinfo(user: UserInfo) -> UserInfoDTO:
    return UserInfoDTO(
        user_id=user.user_id,
        portrait=user.portrait,
        user_name=user.user_name,
        nick_name_new=user.nick_name,
        nick_name_old=user.nick_name_old,
        tieba_uid=user.tieba_uid,
        glevel=user.glevel,
        gender=user.gender.name,
        age=user.age,
        post_num=user.post_num,
        agree_num=user.agree_num,
        fan_num=user.fan_num,
        follow_num=user.follow_num,
        forum_num=user.forum_num,
        sign=user.sign,
        ip=user.ip,
        icons=user.icons,
        is_vip=user.is_vip,
        is_god=user.is_god,
        is_blocked=user.is_blocked,
        priv_like=user.priv_like.name,
        priv_reply=user.priv_reply.name,
    )


@overload
def convert_aiotieba_user(user: UserInfo) -> UserInfoDTO: ...


@overload
def convert_aiotieba_user(user: UserInfo_t) -> ThreadUserDTO:  # type: ignore[overload-cannot-match]
    ...


@overload
def convert_aiotieba_user(user: UserInfo_p) -> PostUserDTO:  # type: ignore[overload-cannot-match]
    ...


@overload
def convert_aiotieba_user(user: UserInfo_c) -> CommentUserDTO:  # type: ignore[overload-cannot-match]
    ...


@overload
def convert_aiotieba_user(user: UserInfo_TUid) -> BaseUserDTO:  # type: ignore[overload-cannot-match]
    ...


def convert_aiotieba_user(
    user: UserInfo_TUid | AiotiebaUserType | UserInfo,
) -> BaseUserDTO | ThreadUserDTO | UserInfoDTO:
    if hasattr(user, "tieba_uid"):
        return convert_aiotieba_userinfo(cast("UserInfo", user))
    if hasattr(user, "ip"):
        return convert_aiotieba_postuser(cast("UserInfo_p", user))
    if hasattr(user, "glevel"):
        return convert_aiotieba_threaduser(cast("UserInfo_t", user))
    if hasattr(user, "is_bawu"):
        return convert_aiotieba_commentuser(cast("UserInfo_c", user))
    return convert_aiotieba_tiebauiduser(cast("UserInfo_TUid", user))


def convert_aiotieba_share_thread(share_thread: ShareThread) -> ShareThreadDTO:
    return ShareThreadDTO(
        pid=share_thread.pid,
        tid=share_thread.tid,
        fid=share_thread.fid,
        fname=share_thread.fname,
        author_id=share_thread.author_id,
        title=share_thread.title,
        contents=convert_aiotieba_content_list(share_thread.contents.objs),
    )


def convert_aiotieba_thread(tb_thread: Thread) -> ThreadDTO:
    """
    将 aiotieba 的 Thread 对象转换为 tiebameow 的通用模型
    """
    return ThreadDTO(
        pid=tb_thread.pid,
        tid=tb_thread.tid,
        fid=tb_thread.fid,
        fname=tb_thread.fname,
        author_id=tb_thread.author_id,
        author=convert_aiotieba_threaduser(tb_thread.user),
        title=tb_thread.title,
        contents=convert_aiotieba_content_list(tb_thread.contents.objs),
        is_good=tb_thread.is_good,
        is_top=tb_thread.is_top,
        is_share=tb_thread.is_share,
        is_hide=tb_thread.is_hide,
        is_livepost=tb_thread.is_livepost,
        is_help=tb_thread.is_help,
        agree_num=tb_thread.agree,
        disagree_num=tb_thread.disagree,
        reply_num=tb_thread.reply_num,
        view_num=tb_thread.view_num,
        share_num=tb_thread.share_num,
        create_time=datetime.fromtimestamp(tb_thread.create_time, SHANGHAI_TZ),
        last_time=datetime.fromtimestamp(tb_thread.last_time, SHANGHAI_TZ),
        thread_type=tb_thread.type,
        tab_id=tb_thread.tab_id,
        share_origin=convert_aiotieba_share_thread(tb_thread.share_origin),
    )


def convert_aiotieba_post(tb_post: Post) -> PostDTO:
    return PostDTO(
        pid=tb_post.pid,
        tid=tb_post.tid,
        fid=tb_post.fid,
        fname=tb_post.fname,
        author_id=tb_post.author_id,
        author=convert_aiotieba_postuser(tb_post.user),
        contents=convert_aiotieba_content_list(tb_post.contents.objs),
        sign=tb_post.sign,
        comments=convert_aiotieba_commentsp(tb_post.comments),
        is_aimeme=tb_post.is_aimeme,
        is_thread_author=tb_post.is_thread_author,
        agree_num=tb_post.agree,
        disagree_num=tb_post.disagree,
        reply_num=tb_post.reply_num,
        create_time=datetime.fromtimestamp(tb_post.create_time, SHANGHAI_TZ),
        floor=tb_post.floor,
    )


def convert_aiotieba_comment(tb_comment: Comment | Comment_p) -> CommentDTO:
    return CommentDTO(
        cid=tb_comment.pid,
        pid=tb_comment.ppid,
        tid=tb_comment.tid,
        fid=tb_comment.fid,
        fname=tb_comment.fname,
        author_id=tb_comment.author_id,
        author=convert_aiotieba_commentuser(tb_comment.user),
        contents=convert_aiotieba_content_list(tb_comment.contents.objs),
        reply_to_id=tb_comment.reply_to_id,
        is_thread_author=tb_comment.is_thread_author,
        agree_num=tb_comment.agree,
        disagree_num=tb_comment.disagree,
        create_time=datetime.fromtimestamp(tb_comment.create_time, SHANGHAI_TZ),
        floor=tb_comment.floor,
    )


def convert_aiotieba_commentsp(tb_post_comments: list[Comment_p]) -> list[CommentDTO]:
    return [convert_aiotieba_comment(tb_comment) for tb_comment in tb_post_comments]


def convert_aiotieba_pageinfo(page: AiotiebaPageType) -> PageInfoDTO:
    return PageInfoDTO(
        page_size=page.page_size,
        current_page=page.current_page,
        total_page=page.total_page,
        total_count=page.total_count,
        has_more=page.has_more,
        has_prev=page.has_prev,
    )


def convert_aiotieba_forum(forum: AiotiebaForumType) -> BaseForumDTO:
    return BaseForumDTO(
        fid=forum.fid,
        fname=forum.fname,
    )


def convert_aiotieba_threads(tb_threads: Threads) -> ThreadsDTO:
    return ThreadsDTO(
        objs=[convert_aiotieba_thread(tb_thread) for tb_thread in tb_threads.objs],
        page=convert_aiotieba_pageinfo(tb_threads.page),
        forum=convert_aiotieba_forum(tb_threads.forum),
    )


def convert_aiotieba_posts(tb_posts: Posts) -> PostsDTO:
    return PostsDTO(
        objs=[convert_aiotieba_post(tb_post) for tb_post in tb_posts.objs],
        page=convert_aiotieba_pageinfo(tb_posts.page),
        forum=convert_aiotieba_forum(tb_posts.forum),
    )


def convert_aiotieba_comments(tb_comments: Comments) -> CommentsDTO:
    return CommentsDTO(
        objs=[convert_aiotieba_comment(tb_comment) for tb_comment in tb_comments.objs],
        page=convert_aiotieba_pageinfo(tb_comments.page),
        forum=convert_aiotieba_forum(tb_comments.forum),
    )



from __future__ import annotations

import asyncio
import ssl
from contextlib import asynccontextmanager
from typing import TYPE_CHECKING, Any

import httpx
from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_exponential_jitter

if TYPE_CHECKING:
    from collections.abc import AsyncGenerator


class HTTPXClient:
    _client: httpx.AsyncClient | None = None
    _context: ssl.SSLContext | None = None
    _lock: asyncio.Lock | None = None

    DEFAULT_TIMEOUT: float = 10.0

    DEFAULT_STOP = stop_after_attempt(3)
    DEFAULT_WAIT = wait_exponential_jitter(initial=0.5, max=3.0)
    DEFAULT_RETRY_CONDITION = retry_if_exception_type((
        httpx.ConnectTimeout,
        httpx.ReadTimeout,
        httpx.RemoteProtocolError,
        httpx.NetworkError,
    ))

    @classmethod
    def _get_lock(cls) -> asyncio.Lock:
        if cls._lock is None:
            cls._lock = asyncio.Lock()
        return cls._lock

    @classmethod
    @asynccontextmanager
    async def get_client(cls) -> AsyncGenerator[httpx.AsyncClient, None]:
        if cls._client is None or cls._client.is_closed:
            async with cls._get_lock():
                if cls._client is None or cls._client.is_closed:
                    if cls._context is None:
                        cls._context = ssl.create_default_context()
                        cls._context.set_ciphers("DEFAULT")
                    cls._client = httpx.AsyncClient(
                        timeout=cls.DEFAULT_TIMEOUT,
                        verify=cls._context,
                    )
        yield cls._client

    @classmethod
    async def close(cls) -> None:
        if cls._client and not cls._client.is_closed:
            await cls._client.aclose()
            cls._client = None

    @classmethod
    def configure_defaults(cls, timeout: float = DEFAULT_TIMEOUT, retry_config: dict[str, Any] | None = None) -> None:
        """
        配置默认的超时和重试策略。

        Args:
            timeout: 默认的请求超时时间（秒）。
            retry_config: 重试配置字典，包含 "stop", "wait", "retry" 键。
        """
        cls.DEFAULT_TIMEOUT = timeout

        if retry_config:
            if "stop" in retry_config:
                cls.DEFAULT_STOP = retry_config["stop"]
            if "wait" in retry_config:
                cls.DEFAULT_WAIT = retry_config["wait"]
            if "retry" in retry_config:
                cls.DEFAULT_RETRY_CONDITION = retry_config["retry"]

    @classmethod
    async def get(cls, url: str, **kwargs: Any) -> httpx.Response | None:
        @retry(stop=cls.DEFAULT_STOP, wait=cls.DEFAULT_WAIT, retry=cls.DEFAULT_RETRY_CONDITION)
        async def _get() -> httpx.Response:
            async with cls.get_client() as client:
                response = await client.get(url, **kwargs)
                response.raise_for_status()
                return response

        try:
            return await _get()
        except httpx.HTTPStatusError:
            return None

    @classmethod
    async def post(cls, url: str, json: dict[str, Any] | None = None, **kwargs: Any) -> httpx.Response | None:
        @retry(stop=cls.DEFAULT_STOP, wait=cls.DEFAULT_WAIT, retry=cls.DEFAULT_RETRY_CONDITION)
        async def _post() -> httpx.Response:
            async with cls.get_client() as client:
                response = await client.post(url, json=json, **kwargs)
                response.raise_for_status()
                return response

        try:
            return await _post()
        except Exception:
            return None



from collections.abc import Iterator
from typing import Any

import pytest
from sqlalchemy import create_engine, select
from sqlalchemy.orm import DeclarativeBase, Mapped, Session, mapped_column, sessionmaker

from tiebameow.models.orm import Fragment, FragmentListType
from tiebameow.schemas.fragments import FragImageModel, FragTextModel


# Define a test model using FragmentListType
class Base(DeclarativeBase):
    pass


class ORMTestModel(Base):
    __tablename__ = "test_model"
    id: Mapped[int] = mapped_column(primary_key=True)
    contents: Mapped[list[Fragment]] = mapped_column(FragmentListType())


@pytest.fixture
def session() -> Iterator[Session]:
    engine = create_engine("sqlite:///:memory:")
    Base.metadata.create_all(engine)
    session_factory = sessionmaker(bind=engine, expire_on_commit=False)
    db = session_factory()
    try:
        yield db
    finally:
        db.close()


def test_fragment_list_type(session: Session) -> None:
    # Create data
    fragments = [
        FragTextModel(text="hello"),
        FragImageModel(
            src="src", big_src="big", origin_src="origin", origin_size=100, show_width=100, show_height=100, hash="hash"
        ),
    ]
    obj = ORMTestModel(contents=fragments)
    session.add(obj)
    session.commit()

    # Read data
    loaded_obj = session.execute(select(ORMTestModel)).scalar_one()
    assert len(loaded_obj.contents) == 2
    assert isinstance(loaded_obj.contents[0], FragTextModel)
    assert loaded_obj.contents[0].text == "hello"
    assert isinstance(loaded_obj.contents[1], FragImageModel)
    assert loaded_obj.contents[1].src == "src"


def test_fragment_list_type_empty(session: Session) -> None:
    obj = ORMTestModel(contents=[])
    session.add(obj)
    session.commit()

    loaded_obj = session.execute(select(ORMTestModel)).scalar_one()
    assert loaded_obj.contents == []


def test_fragment_list_type_none(session: Session) -> None:
    # Test with None if allowed by model (though mapped_column usually implies not null unless nullable=True)
    # Here we just test the type behavior if we were to pass None to process_bind_param manually
    type_impl = FragmentListType()
    dialect: Any = None
    assert type_impl.process_bind_param(None, dialect) is None
    assert type_impl.process_result_value(None, dialect) is None



from .serializer import (
    deserialize,
    deserialize_comment,
    deserialize_post,
    deserialize_thread,
    serialize,
)

__all__ = [
    "serialize",
    "deserialize",
    "deserialize_thread",
    "deserialize_post",
    "deserialize_comment",
]



from datetime import datetime
from zoneinfo import ZoneInfo

__all__ = ["SHANGHAI_TZ", "now_with_tz"]

SHANGHAI_TZ = ZoneInfo("Asia/Shanghai")


def now_with_tz() -> datetime:
    """返回带时区的当前时间。

    Returns:
        datetime: 上海时区的当前时间。
    """
    return datetime.now(SHANGHAI_TZ)



from typing import Any

from tiebameow.parser.parser import (
    convert_aiotieba_content_list,
    convert_aiotieba_fragment,
    convert_aiotieba_threaduser,
    convert_aiotieba_tiebauiduser,
    convert_aiotieba_userinfo,
)
from tiebameow.schemas.fragments import (
    FragAtModel,
    FragEmojiModel,
    FragImageModel,
    FragItemModel,
    FragLinkModel,
    FragTextModel,
    FragUnknownModel,
)


class MockUser:
    def __init__(self, **kwargs: Any) -> None:
        for k, v in kwargs.items():
            setattr(self, k, v)


def test_convert_aiotieba_fragment(mock_aiotieba_fragments: dict[str, Any]) -> None:
    # Text
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["text"])
    assert isinstance(res, FragTextModel)
    assert res.text == "hello"

    # Image
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["image"])
    assert isinstance(res, FragImageModel)
    assert res.src == "http://src"

    # At
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["at"])
    assert isinstance(res, FragAtModel)
    assert res.text == "@user"

    # Link
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["link"])
    assert isinstance(res, FragLinkModel)
    assert res.text == "http://link"

    # Emoji
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["emoji"])
    assert isinstance(res, FragEmojiModel)
    assert res.id == "1"

    # Item
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["item"])
    assert isinstance(res, FragItemModel)
    assert res.text == "item"

    # Unknown
    res = convert_aiotieba_fragment(mock_aiotieba_fragments["unknown"])
    assert isinstance(res, FragUnknownModel)


def test_convert_aiotieba_content_list(mock_aiotieba_fragments: dict[str, Any]) -> None:
    contents = [mock_aiotieba_fragments["text"], mock_aiotieba_fragments["image"]]
    res = convert_aiotieba_content_list(contents)
    assert len(res) == 2
    assert isinstance(res[0], FragTextModel)
    assert isinstance(res[1], FragImageModel)


def test_convert_aiotieba_tiebauiduser() -> None:
    user = MockUser(user_id=1, portrait="portrait", user_name="user_name", nick_name="nick_name")
    res = convert_aiotieba_tiebauiduser(user)
    assert res.user_id == 1
    assert res.nick_name == "nick_name"


def test_convert_aiotieba_threaduser() -> None:
    user = MockUser(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name="nick_name",
        level=1,
        glevel=1,
        gender=MockUser(name="MALE"),  # Mock enum
        ip="127.0.0.1",
        icons=[],
        is_bawu=False,
        is_vip=False,
        is_god=False,
        priv_like=MockUser(name="PUBLIC"),
        priv_reply=MockUser(name="ALL"),
    )
    res = convert_aiotieba_threaduser(user)
    assert res.user_id == 1
    assert res.gender == "MALE"


def test_convert_aiotieba_userinfo() -> None:
    user = MockUser(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name="nick_name",
        nick_name_old="old",
        tieba_uid=123,
        glevel=1,
        gender=MockUser(name="MALE"),
        age=1.0,
        post_num=10,
        agree_num=10,
        fan_num=10,
        follow_num=10,
        forum_num=10,
        sign="sign",
        ip="127.0.0.1",
        icons=[],
        is_vip=False,
        is_god=False,
        is_blocked=False,
        priv_like=MockUser(name="PUBLIC"),
        priv_reply=MockUser(name="ALL"),
    )
    res = convert_aiotieba_userinfo(user)
    assert res.user_id == 1
    assert res.post_num == 10



from tiebameow.models.dto import CommentDTO, PostDTO, ThreadDTO
from tiebameow.schemas.fragments import FragTextModel
from tiebameow.serializer.serializer import (
    deserialize_comment,
    deserialize_post,
    deserialize_thread,
    serialize,
)


def test_serialize() -> None:
    model = FragTextModel(text="hello")
    res = serialize(model)
    assert res == {"type": "text", "text": "hello"}

    # Test non-model object
    assert serialize("string") == "string"


def test_deserialize_thread() -> None:
    data = {
        "pid": 1,
        "tid": 1,
        "fid": 1,
        "fname": "fname",
        "author_id": 1,
        "author": {
            "user_id": 1,
            "portrait": "p",
            "user_name": "u",
            "nick_name_new": "n",
            "level": 1,
            "glevel": 1,
            "gender": "MALE",
            "icons": [],
            "is_bawu": False,
            "is_vip": False,
            "is_god": False,
            "priv_like": "PUBLIC",
            "priv_reply": "ALL",
        },
        "title": "title",
        "contents": [{"type": "text", "text": "content"}],
        "is_good": False,
        "is_top": False,
        "is_share": False,
        "is_hide": False,
        "is_livepost": False,
        "is_help": False,
        "agree_num": 0,
        "disagree_num": 0,
        "reply_num": 0,
        "view_num": 0,
        "share_num": 0,
        "create_time": "2023-01-01T00:00:00",
        "last_time": "2023-01-01T00:00:00",
        "thread_type": 0,
        "tab_id": 0,
        "share_origin": {"pid": 0, "tid": 0, "fid": 0, "fname": "", "author_id": 0, "title": "", "contents": []},
    }
    thread = deserialize_thread(data)
    assert isinstance(thread, ThreadDTO)
    assert thread.title == "title"
    assert len(thread.contents) == 1


def test_deserialize_thread_normalization() -> None:
    # Test contents normalization (objs wrapper) and user normalization (user -> author)
    data = {
        "pid": 1,
        "tid": 1,
        "fid": 1,
        "fname": "fname",
        "author_id": 1,
        "user": {  # Should be mapped to author
            "user_id": 1,
            "portrait": "p",
            "user_name": "u",
            "nick_name_new": "n",
            "level": 1,
            "glevel": 1,
            "gender": "MALE",
            "icons": [],
            "is_bawu": False,
            "is_vip": False,
            "is_god": False,
            "priv_like": "PUBLIC",
            "priv_reply": "ALL",
        },
        "title": "title",
        "contents": {"objs": [{"type": "text", "text": "content"}]},  # Should be unwrapped
        "is_good": False,
        "is_top": False,
        "is_share": False,
        "is_hide": False,
        "is_livepost": False,
        "is_help": False,
        "agree_num": 0,
        "disagree_num": 0,
        "reply_num": 0,
        "view_num": 0,
        "share_num": 0,
        "create_time": "2023-01-01T00:00:00",
        "last_time": "2023-01-01T00:00:00",
        "thread_type": 0,
        "tab_id": 0,
        "share_origin": {"pid": 0, "tid": 0, "fid": 0, "fname": "", "author_id": 0, "title": "", "contents": []},
    }
    thread = deserialize_thread(data)
    assert thread.author.user_id == 1
    assert len(thread.contents) == 1


def test_deserialize_post() -> None:
    data = {
        "pid": 1,
        "tid": 1,
        "fid": 1,
        "fname": "fname",
        "author_id": 1,
        "author": {
            "user_id": 1,
            "portrait": "p",
            "user_name": "u",
            "nick_name_new": "n",
            "level": 1,
            "glevel": 1,
            "gender": "MALE",
            "ip": "127.0.0.1",
            "icons": [],
            "is_bawu": False,
            "is_vip": False,
            "is_god": False,
            "priv_like": "PUBLIC",
            "priv_reply": "ALL",
        },
        "contents": [{"type": "text", "text": "content"}],
        "sign": "",
        "comments": [
            {
                "cid": 1,
                "pid": 1,
                "tid": 1,
                "fid": 1,
                "fname": "fname",
                "author_id": 1,
                "user": {
                    "user_id": 1,
                    "portrait": "p",
                    "user_name": "u",
                    "nick_name_new": "n",
                    "level": 1,
                    "gender": "MALE",
                    "icons": [],
                    "is_bawu": False,
                    "is_vip": False,
                    "is_god": False,
                    "priv_like": "PUBLIC",
                    "priv_reply": "ALL",
                },
                "contents": [{"type": "text", "text": "content"}],
                "reply_to_id": 0,
                "is_thread_author": False,
                "agree_num": 0,
                "disagree_num": 0,
                "create_time": "2023-01-01T00:00:00",
                "floor": 1,
            }
        ],
        "is_aimeme": False,
        "is_thread_author": False,
        "agree_num": 0,
        "disagree_num": 0,
        "reply_num": 0,
        "create_time": "2023-01-01T00:00:00",
        "floor": 1,
    }
    post = deserialize_post(data)
    assert isinstance(post, PostDTO)
    assert post.pid == 1
    assert post.author.user_id == 1
    assert len(post.contents) == 1
    assert isinstance(post.contents[0], FragTextModel)


def test_deserialize_comment() -> None:
    data = {
        "cid": 1,
        "pid": 1,
        "tid": 1,
        "fid": 1,
        "fname": "fname",
        "author_id": 1,
        "author": {
            "user_id": 1,
            "portrait": "p",
            "user_name": "u",
            "nick_name_new": "n",
            "level": 1,
            "gender": "MALE",
            "icons": [],
            "is_bawu": False,
            "is_vip": False,
            "is_god": False,
            "priv_like": "PUBLIC",
            "priv_reply": "ALL",
        },
        "contents": [{"type": "text", "text": "content"}],
        "reply_to_id": 0,
        "is_thread_author": False,
        "agree_num": 0,
        "disagree_num": 0,
        "create_time": "2023-01-01T00:00:00",
        "floor": 1,
    }
    comment = deserialize_comment(data)
    assert isinstance(comment, CommentDTO)
    assert comment.cid == 1



from __future__ import annotations

import asyncio
from typing import Any, NamedTuple
from unittest.mock import AsyncMock, patch

import pytest
from aiotieba.exception import HTTPStatusError
from tenacity import AsyncRetrying, retry_if_exception_type, stop_after_attempt, wait_none

from tiebameow.client.tieba_client import Client, RetriableApiError, UnretriableApiError, with_ensure


class _Result(NamedTuple):
    err: object | None = None


class _AsyncCM:
    def __init__(self) -> None:
        self.entered = 0
        self.exited = 0

    async def __aenter__(self) -> None:
        self.entered += 1

    async def __aexit__(self, exc_type: object, exc: object, tb: object) -> None:
        self.exited += 1


def _set_no_wait_retry(client: Client, *, attempts: int = 3) -> None:
    client._retry_strategy = AsyncRetrying(
        stop=stop_after_attempt(attempts),
        wait=wait_none(),
        retry=retry_if_exception_type((
            OSError,
            asyncio.TimeoutError,
            ConnectionError,
            RetriableApiError,
        )),
        reraise=True,
    )


@pytest.mark.asyncio
async def test_client_init() -> None:
    client = Client()
    assert client._limiter is None
    assert client._semaphore is None
    assert client._cooldown_429 == 0.0


@pytest.mark.asyncio
async def test_client_context_manager() -> None:
    client = Client()
    with (
        patch("tiebameow.client.tieba_client.tb.Client.__aenter__", new_callable=AsyncMock) as mock_aenter,
        patch("tiebameow.client.tieba_client.tb.Client.__aexit__", new_callable=AsyncMock) as mock_aexit,
    ):
        async with client as c:
            assert c is client

        mock_aenter.assert_awaited_once()
        mock_aexit.assert_awaited_once()


@pytest.mark.asyncio
async def test_with_limits_enters_limiter_and_semaphore() -> None:
    limiter = _AsyncCM()
    semaphore = _AsyncCM()
    client = Client(limiter=limiter, semaphore=semaphore)

    async with client._with_limits():
        pass

    assert limiter.entered == 1
    assert limiter.exited == 1
    assert semaphore.entered == 1
    assert semaphore.exited == 1


@pytest.mark.asyncio
async def test_with_ensure_retry_success() -> None:
    client = Client()
    _set_no_wait_retry(client)

    mock_func = AsyncMock(return_value="success")

    @with_ensure
    async def decorated_func(self: Client, *args: Any, **kwargs: Any) -> Any:
        return await mock_func(self, *args, **kwargs)

    res = await decorated_func(client)
    assert res == "success"
    assert mock_func.call_count == 1


@pytest.mark.asyncio
async def test_with_ensure_retry_fail_then_success() -> None:
    client = Client()
    _set_no_wait_retry(client, attempts=2)

    mock_func = AsyncMock(side_effect=[TimeoutError("timeout"), "success"])

    @with_ensure
    async def decorated_func(self: Client, *args: Any, **kwargs: Any) -> Any:
        return await mock_func(self, *args, **kwargs)

    res = await decorated_func(client)
    assert res == "success"
    assert mock_func.call_count == 2


@pytest.mark.asyncio
async def test_request_core_sets_global_cooldown_on_429() -> None:
    client = Client(cooldown_429=0.1)
    _set_no_wait_retry(client, attempts=2)

    # 第一次返回携带 err=429 的结果，触发 RetriableApiError(429) 并设置全局冷却；第二次正常。
    err_429 = HTTPStatusError(429, "Too Many Requests")
    mock_func = AsyncMock(side_effect=[_Result(err_429), _Result(None)])

    with (
        patch("tiebameow.client.tieba_client.asyncio.sleep", new_callable=AsyncMock) as mock_sleep,
        patch("tiebameow.client.tieba_client.time.monotonic", new=lambda: 0.0),
    ):

        async def call(self: Client) -> Any:
            return await mock_func(self)

        res = await client._request_core(call)

    assert isinstance(res, _Result)
    assert mock_func.call_count == 2
    mock_sleep.assert_any_await(0.1)


@pytest.mark.asyncio
async def test_request_core_critical_error_no_retry() -> None:
    client = Client()
    _set_no_wait_retry(client, attempts=3)

    err_999 = HTTPStatusError(999, "Critical")
    mock_func = AsyncMock(return_value=_Result(err_999))

    async def call(self: Client) -> Any:
        return await mock_func(self)

    with pytest.raises(UnretriableApiError):
        await client._request_core(call)

    assert mock_func.call_count == 1


@pytest.mark.asyncio
async def test_wrapped_get_threads_calls_super() -> None:
    client = Client()
    _set_no_wait_retry(client)

    with patch("tiebameow.client.tieba_client.tb.Client.get_threads", new_callable=AsyncMock) as mock_super:
        mock_super.return_value = "threads"
        res = await client.get_threads("test", 1, rn=30)

    assert res == "threads"
    mock_super.assert_awaited_once()



from __future__ import annotations

import asyncio
import time
from collections.abc import Awaitable, Callable
from contextlib import AsyncExitStack, asynccontextmanager
from functools import wraps
from typing import TYPE_CHECKING, Any

import aiotieba as tb
from aiotieba.exception import BoolResponse, HTTPStatusError, IntResponse, StrResponse, TiebaServerError
from tenacity import (
    AsyncRetrying,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential_jitter,
)

from ..parser import (
    convert_aiotieba_comments,
    convert_aiotieba_posts,
    convert_aiotieba_threads,
    convert_aiotieba_userinfo,
)
from ..utils.logger import logger

if TYPE_CHECKING:
    from collections.abc import AsyncGenerator
    from datetime import datetime

    from aiolimiter import AsyncLimiter
    from aiotieba.api.get_bawu_blacklist._classdef import BawuBlacklistUsers
    from aiotieba.api.get_bawu_postlogs._classdef import Postlogs
    from aiotieba.api.get_bawu_userlogs._classdef import Userlogs
    from aiotieba.api.get_follow_forums._classdef import FollowForums
    from aiotieba.api.get_tab_map._classdef import TabMap
    from aiotieba.api.get_user_contents._classdef import UserPostss, UserThreads
    from aiotieba.api.tieba_uid2user_info._classdef import UserInfo_TUid
    from aiotieba.typing import Comments, Posts, Threads, UserInfo

    from ..models.dto import CommentsDTO, PostsDTO, ThreadsDTO, UserInfoDTO


class AiotiebaError(Exception):
    """基础 aiotieba API 异常"""

    def __init__(self, code: int, msg: str):
        self.code = code
        self.msg = msg
        super().__init__(f"[{code}] {msg}")


class RetriableApiError(AiotiebaError):
    """aiotieba 返回的可重试的异常"""


class UnretriableApiError(AiotiebaError):
    """aiotieba 返回的无法重试的异常"""


class ErrorHandler:
    RETRIABLE_CODES: set[int] = {
        -65536,
        11,
        77,
        408,
        429,
        4011,
        110001,
        220034,
        230871,
        300000,
        1989005,
        2210002,
        28113295,
    }

    @classmethod
    def check(cls, result: Any) -> None:
        """解析 aiotieba 返回对象中的 err 字段"""
        err = getattr(result, "err", None)
        if err is None:
            return

        if isinstance(err, (HTTPStatusError, TiebaServerError)):
            code = err.code
            msg = err.msg

            if code in cls.RETRIABLE_CODES:
                raise RetriableApiError(code, msg)

            raise UnretriableApiError(code, msg)

        elif isinstance(err, Exception):
            raise err


def with_ensure[F: Callable[..., Awaitable[Any]]](func: F) -> F:
    """装饰器：为 aiotieba.Client 的方法添加重试和限流支持。"""

    @wraps(func)
    async def wrapper(self: Client, *args: Any, **kwargs: Any) -> Any:
        return await self._request_core(func, *args, **kwargs)

    return wrapper  # type: ignore[return-value]


class Client(tb.Client):  # type: ignore[misc]
    """扩展的aiotieba客户端，添加了自定义的请求限流和并发控制功能。

    该客户端继承自aiotieba.Client，并在其基础上实现了速率限制和并发控制。
    通过装饰器和上下文管理器的方式，为所有API调用提供统一的速率限制和并发控制。
    同时还添加了对特定错误码的重试机制，以提高请求的成功率。
    """

    def __init__(
        self,
        *args: Any,
        limiter: AsyncLimiter | None = None,
        semaphore: asyncio.Semaphore | None = None,
        cooldown_429: float = 0.0,
        retry_attempts: int = 3,
        wait_initial: float = 0.5,
        wait_max: float = 5.0,
        **kwargs: Any,
    ):
        """初始化扩展的aiotieba客户端。

        Args:
            *args: 传递给父类构造函数的参数。
            limiter: 速率限制器，用于控制每秒请求数。
            semaphore: 信号量，用于控制最大并发数。
            cooldown_seconds: 触发429时的全局冷却秒数。
            **kwargs: 传递给父类构造函数的关键字参数。
        """
        super().__init__(*args, **kwargs)
        self._limiter = limiter
        self._semaphore = semaphore
        self._cooldown_429 = cooldown_429
        self._cooldown_until: float = 0.0

        self._retry_strategy = AsyncRetrying(
            stop=stop_after_attempt(retry_attempts),
            wait=wait_exponential_jitter(initial=wait_initial, max=wait_max),
            retry=retry_if_exception_type((
                OSError,
                asyncio.TimeoutError,
                ConnectionError,
                HTTPStatusError,
                TiebaServerError,
                RetriableApiError,
            )),
            reraise=True,
        )

    async def __aenter__(self) -> Client:
        await super().__aenter__()
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None = None,
        exc_val: BaseException | None = None,
        exc_tb: object = None,
    ) -> None:
        await super().__aexit__(exc_type, exc_val, exc_tb)

    @property
    def limiter(self) -> AsyncLimiter | None:
        """获取速率限制器。"""
        return self._limiter

    @property
    def semaphore(self) -> asyncio.Semaphore | None:
        """获取信号量。"""
        return self._semaphore

    @asynccontextmanager
    async def _with_limits(self) -> AsyncGenerator[None, None]:
        """内部限流上下文管理器"""
        async with AsyncExitStack() as stack:
            if self._limiter:
                await stack.enter_async_context(self._limiter)
            if self._semaphore:
                await stack.enter_async_context(self._semaphore)
            yield

    async def _request_core(self, func: Callable[..., Awaitable[Any]], *args: Any, **kwargs: Any) -> Any:
        """核心调度逻辑，处理限流、熔断和错误转换"""
        async for attempt in self._retry_strategy:
            with attempt:
                now = time.monotonic()
                if now < self._cooldown_until:
                    wait_time = self._cooldown_until - now
                    logger.debug("Global cooldown active. Waiting for {:.1f}s", wait_time)
                    await asyncio.sleep(wait_time)

                async with self._with_limits():
                    result = await func(self, *args, **kwargs)

                try:
                    ErrorHandler.check(result)
                except RetriableApiError as e:
                    if e.code == 429 and self._cooldown_429 > 0:
                        self._cooldown_until = time.monotonic() + self._cooldown_429
                    logger.warning("Retrying {} due to: {}", func.__name__, e)
                    raise
                except UnretriableApiError:
                    raise

                return result

    # 以下为直接返回DTO模型的封装方法

    # 获取贴子内容 #

    async def get_threads_dto(
        self,
        fname_or_fid: str | int,
        /,
        pn: int = 1,
        *,
        rn: int = 30,
        sort: tb.ThreadSortType = tb.ThreadSortType.REPLY,
        is_good: bool = False,
    ) -> ThreadsDTO:
        """获取指定贴吧的主题列表，并转换为通用DTO模型。"""
        threads = await self.get_threads(fname_or_fid, pn, rn=rn, sort=sort, is_good=is_good)
        return convert_aiotieba_threads(threads)

    async def get_posts_dto(
        self,
        tid: int,
        /,
        pn: int = 1,
        *,
        rn: int = 30,
        sort: tb.PostSortType = tb.PostSortType.ASC,
        only_thread_author: bool = False,
        with_comments: bool = False,
        comment_sort_by_agree: bool = True,
        comment_rn: int = 4,
    ) -> PostsDTO:
        """获取指定主题贴的回复列表，并转换为通用DTO模型。"""
        posts = await self.get_posts(
            tid,
            pn,
            rn=rn,
            sort=sort,
            only_thread_author=only_thread_author,
            with_comments=with_comments,
            comment_sort_by_agree=comment_sort_by_agree,
            comment_rn=comment_rn,
        )
        return convert_aiotieba_posts(posts)

    async def get_comments_dto(
        self,
        tid: int,
        pid: int,
        /,
        pn: int = 1,
        *,
        is_comment: bool = False,
    ) -> CommentsDTO:
        """获取指定回复的楼中楼列表，并转换为通用DTO模型。"""
        comments = await self.get_comments(tid, pid, pn, is_comment=is_comment)
        return convert_aiotieba_comments(comments)

    # 获取用户信息 #

    async def anyid2user_info_dto(self, uid: int | str, is_tieba_uid: bool = True) -> UserInfoDTO:
        """
        根据任意用户ID获取完整的用户信息，并转换为通用DTO模型。

        Args:
            uid: 用户ID，可以是贴吧ID、user_id、portrait或用户名。
            is_tieba_uid: 指示uid是否为贴吧UID，默认为True。
        """
        if is_tieba_uid and isinstance(uid, int):
            user_tuid = await self.tieba_uid2user_info(uid)
            user = await self.get_user_info(user_tuid.user_id)
        else:
            user = await self.get_user_info(uid)
        return convert_aiotieba_userinfo(user)

    async def get_nickname_old(self, user_id: int) -> str:
        user_info = await self.get_user_info(user_id, require=tb.ReqUInfo.BASIC)
        return str(user_info.nick_name_old)

    # 以下为重写的部分 aiotieba.Client API
    # 添加了 @with_ensure 装饰器以启用重试机制
    # 完全拦截过于魔法，这里仅重写部分常用API

    # 获取贴子内容 #

    @with_ensure
    async def get_threads(
        self,
        fname_or_fid: str | int,
        /,
        pn: int = 1,
        *,
        rn: int = 30,
        sort: tb.ThreadSortType = tb.ThreadSortType.REPLY,
        is_good: bool = False,
    ) -> Threads:
        return await super().get_threads(fname_or_fid, pn, rn=rn, sort=sort, is_good=is_good)

    @with_ensure
    async def get_posts(
        self,
        tid: int,
        /,
        pn: int = 1,
        *,
        rn: int = 30,
        sort: tb.PostSortType = tb.PostSortType.ASC,
        only_thread_author: bool = False,
        with_comments: bool = False,
        comment_sort_by_agree: bool = True,
        comment_rn: int = 4,
    ) -> Posts:
        return await super().get_posts(
            tid,
            pn,
            rn=rn,
            sort=sort,
            only_thread_author=only_thread_author,
            with_comments=with_comments,
            comment_sort_by_agree=comment_sort_by_agree,
            comment_rn=comment_rn,
        )

    @with_ensure
    async def get_comments(
        self,
        tid: int,
        pid: int,
        /,
        pn: int = 1,
        *,
        is_comment: bool = False,
    ) -> Comments:
        return await super().get_comments(tid, pid, pn, is_comment=is_comment)

    @with_ensure
    async def get_user_threads(
        self,
        id_: str | int | None = None,
        pn: int = 1,
        *,
        public_only: bool = False,
    ) -> UserThreads:
        return await super().get_user_threads(id_, pn, public_only=public_only)

    @with_ensure
    async def get_user_posts(
        self,
        id_: str | int | None = None,
        pn: int = 1,
        *,
        rn: int = 20,
    ) -> UserPostss:
        return await super().get_user_posts(id_, pn, rn=rn)

    # 获取用户信息 #

    @with_ensure
    async def tieba_uid2user_info(self, tieba_uid: int) -> UserInfo_TUid:
        return await super().tieba_uid2user_info(tieba_uid)

    @with_ensure
    async def get_user_info(self, id_: str | int, /, require: tb.ReqUInfo = tb.ReqUInfo.ALL) -> UserInfo:
        return await super().get_user_info(id_, require)

    @with_ensure
    async def get_self_info(self, require: tb.ReqUInfo = tb.ReqUInfo.ALL) -> UserInfo:
        return await super().get_self_info(require)

    @with_ensure
    async def get_follow_forums(self, id_: str | int, /, pn: int = 1, *, rn: int = 50) -> FollowForums:
        return await super().get_follow_forums(id_, pn, rn=rn)

    # 获取贴吧信息 #

    @with_ensure
    async def get_fid(self, fname: str) -> IntResponse:
        return await super().get_fid(fname)

    @with_ensure
    async def get_fname(self, fid: int) -> StrResponse:
        return await super().get_fname(fid)

    @with_ensure
    async def get_tab_map(self, fname_or_fid: str | int) -> TabMap:
        return await super().get_tab_map(fname_or_fid)

    # 吧务查询 #

    @with_ensure
    async def get_bawu_blacklist(self, fname_or_fid: str | int, /, pn: int = 1) -> BawuBlacklistUsers:
        return await super().get_bawu_blacklist(fname_or_fid, pn)

    @with_ensure
    async def get_bawu_postlogs(
        self,
        fname_or_fid: str | int,
        /,
        pn: int = 1,
        *,
        search_value: str = "",
        search_type: tb.BawuSearchType = tb.BawuSearchType.USER,
        start_dt: datetime | None = None,
        end_dt: datetime | None = None,
        op_type: int = 0,
    ) -> Postlogs:
        return await super().get_bawu_postlogs(
            fname_or_fid,
            pn,
            search_value=search_value,
            search_type=search_type,
            start_dt=start_dt,
            end_dt=end_dt,
            op_type=op_type,
        )

    @with_ensure
    async def get_bawu_userlogs(
        self,
        fname_or_fid: str | int,
        /,
        pn: int = 1,
        *,
        search_value: str = "",
        search_type: tb.BawuSearchType = tb.BawuSearchType.USER,
        start_dt: datetime | None = None,
        end_dt: datetime | None = None,
        op_type: int = 0,
    ) -> Userlogs:
        return await super().get_bawu_userlogs(
            fname_or_fid,
            pn,
            search_value=search_value,
            search_type=search_type,
            start_dt=start_dt,
            end_dt=end_dt,
            op_type=op_type,
        )

    # 吧务操作 #

    @with_ensure
    async def del_thread(self, fname_or_fid: str | int, /, tid: int) -> BoolResponse:
        return await super().del_thread(fname_or_fid, tid)

    @with_ensure
    async def del_post(self, fname_or_fid: str | int, /, tid: int, pid: int) -> BoolResponse:
        return await super().del_post(fname_or_fid, tid, pid)

    @with_ensure
    async def add_bawu_blacklist(self, fname_or_fid: str | int, /, id_: str | int) -> BoolResponse:
        return await super().add_bawu_blacklist(fname_or_fid, id_)

    @with_ensure
    async def del_bawu_blacklist(self, fname_or_fid: str | int, /, id_: str | int) -> BoolResponse:
        return await super().del_bawu_blacklist(fname_or_fid, id_)

    @with_ensure
    async def block(
        self, fname_or_fid: str | int, /, id_: str | int, *, day: int = 1, reason: str = ""
    ) -> BoolResponse:
        return await super().block(fname_or_fid, id_, day=day, reason=reason)

    @with_ensure
    async def unblock(self, fname_or_fid: str | int, /, id_: str | int) -> BoolResponse:
        return await super().unblock(fname_or_fid, id_)

    @with_ensure
    async def good(self, fname_or_fid: str | int, /, tid: int, *, cname: str = "") -> BoolResponse:
        return await super().good(fname_or_fid, tid, cname=cname)

    @with_ensure
    async def ungood(self, fname_or_fid: str | int, /, tid: int) -> BoolResponse:
        return await super().ungood(fname_or_fid, tid)

    @with_ensure
    async def top(self, fname_or_fid: str | int, /, tid: int, *, is_vip: bool = False) -> BoolResponse:
        return await super().top(fname_or_fid, tid, is_vip=is_vip)

    @with_ensure
    async def untop(self, fname_or_fid: str | int, /, tid: int, *, is_vip: bool = False) -> BoolResponse:
        return await super().untop(fname_or_fid, tid, is_vip=is_vip)



from .parser import (
    convert_aiotieba_comment,
    convert_aiotieba_comments,
    convert_aiotieba_commentsp,
    convert_aiotieba_commentuser,
    convert_aiotieba_content_list,
    convert_aiotieba_forum,
    convert_aiotieba_fragment,
    convert_aiotieba_pageinfo,
    convert_aiotieba_post,
    convert_aiotieba_posts,
    convert_aiotieba_postuser,
    convert_aiotieba_share_thread,
    convert_aiotieba_thread,
    convert_aiotieba_threads,
    convert_aiotieba_threaduser,
    convert_aiotieba_tiebauiduser,
    convert_aiotieba_user,
    convert_aiotieba_userinfo,
)

__all__ = [
    "convert_aiotieba_comment",
    "convert_aiotieba_comments",
    "convert_aiotieba_commentsp",
    "convert_aiotieba_commentuser",
    "convert_aiotieba_content_list",
    "convert_aiotieba_forum",
    "convert_aiotieba_fragment",
    "convert_aiotieba_pageinfo",
    "convert_aiotieba_post",
    "convert_aiotieba_posts",
    "convert_aiotieba_postuser",
    "convert_aiotieba_share_thread",
    "convert_aiotieba_thread",
    "convert_aiotieba_threads",
    "convert_aiotieba_threaduser",
    "convert_aiotieba_tiebauiduser",
    "convert_aiotieba_user",
    "convert_aiotieba_userinfo",
]



import logging
import sys
from pathlib import Path
from typing import TYPE_CHECKING

from loguru import logger

if TYPE_CHECKING:
    from types import FrameType

__all__ = ["logger", "init_logger"]


class InterceptHandler(logging.Handler):
    """
    拦截标准日志消息并将其路由到 Loguru。
    """

    def emit(self, record: logging.LogRecord) -> None:
        try:
            level: str | int = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        frame: FrameType | None = logging.currentframe()
        depth = 2
        while frame and frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


def init_logger(
    *,
    service_name: str = "tiebameow",
    level: str = "INFO",
    console_format: str | None = None,
    enable_filelog: bool = False,
    enable_error_filelog: bool = False,
    log_dir: str | Path = "logs",
    file_format: str | None = None,
    rotation: str = "00:00",
    retention: str = "14 days",
    intercept_standard_logging: bool = True,
    enqueue: bool = True,
    diagnose: bool = True,
    reset: bool = True,
    add_console: bool = True,
) -> None:
    """
    使用标准配置初始化适用于 TiebaMeow 服务的 loguru logger。

    Args:
        service_name: 服务名称，用于日志文件命名。

        level: 最低日志级别（例如 "DEBUG", "INFO"）。
        console_format: 控制台输出的自定义格式。
        enable_filelog: 是否启用日志文件记录。
        enable_error_filelog: 是否启用错误日志文件记录。
        log_dir: 日志文件存储目录。
        file_format: 文件输出的自定义格式。
        rotation: 日志轮转条件（例如 "1 day", "500 MB", "00:00"）。
        retention: 日志保留条件（例如 "10 days"）。
        intercept_standard_logging: 是否拦截标准库日志。
        enqueue: 是否使用线程安全的日志记录（异步安全）。
        diagnose: 是否在异常回溯中显示变量值。
        reset: 是否移除所有现有的处理器。如果集成到已有日志记录的应用中（例如 NoneBot），请设置为 False。
        add_console: 是否添加控制台处理器。如果应用已经有一个，请设置为 False。
    """
    if console_format is None:
        console_format = (
            "<green>{time:YYYY-MM-DD HH:mm:ss}</green> "
            "<level>[{level}]</level> "
            "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> | "
            "<level>{message}</level>"
        )

    if reset:
        logger.remove()

    if add_console:
        logger.add(
            sys.stderr,
            format=console_format,
            level=level,
            enqueue=enqueue,
            diagnose=diagnose,
        )

    if enable_filelog or enable_error_filelog:
        if file_format is None:
            file_format = "{time:YYYY-MM-DD HH:mm:ss} [{level}] {name}:{function}:{line} | {message}"

        log_path = Path(log_dir)
        try:
            log_path.mkdir(parents=True, exist_ok=True)
        except Exception:
            logger.warning("Failed to create log directory: {}, file logging disabled.", log_path)
            return

        if enable_filelog:
            logger.add(
                log_path / f"{service_name}.log",
                format=file_format,
                level=level,
                rotation=rotation,
                retention=retention,
                encoding="utf-8",
                enqueue=enqueue,
                compression="zip",
                diagnose=diagnose,
            )

        if enable_error_filelog:
            logger.add(
                log_path / f"{service_name}.error.log",
                format=file_format,
                level="ERROR",
                rotation=rotation,
                retention=retention,
                encoding="utf-8",
                enqueue=enqueue,
                compression="zip",
                diagnose=diagnose,
            )

    if intercept_standard_logging:
        logging.basicConfig(handlers=[InterceptHandler()], level=0, force=True)



from typing import Literal, Protocol, get_args, runtime_checkable

from pydantic import BaseModel, field_validator


class FragAtModel(BaseModel):
    """@碎片模型。

    Attributes:
        type: 片段类型，固定为'at'
        text (str): 被@用户的昵称 含@
        user_id (int): 被@用户的user_id
    """

    type: Literal["at"] = "at"
    text: str = ""
    user_id: int = 0


class FragEmojiModel(BaseModel):
    """表情碎片模型。

    Attributes:
        type: 片段类型，固定为'emoji'
        id (str): 表情图片id
        desc (str): 表情描述
    """

    type: Literal["emoji"] = "emoji"
    id: str = ""
    desc: str = ""


class FragImageModel(BaseModel):
    """图像碎片模型。

    Attributes:
        type: 片段类型，固定为'image'
        src (str): 小图链接 宽720px
        big_src (str): 大图链接 宽960px
        origin_src (str): 原图链接
        origin_size (int): 原图大小（字节）
        show_width (int): 图像在客户端预览显示的宽度（像素）
        show_height (int): 图像在客户端预览显示的高度（像素）
        hash (str): 百度图床hash
    """

    type: Literal["image"] = "image"
    src: str = ""
    big_src: str = ""
    origin_src: str = ""
    origin_size: int = 0
    show_width: int = 0
    show_height: int = 0
    hash: str = ""


class FragItemModel(BaseModel):
    """item碎片模型。

    Attributes:
        type: 片段类型，固定为'item'
        text (str): item名称
    """

    type: Literal["item"] = "item"
    text: str = ""


class FragLinkModel(BaseModel):
    """链接碎片模型。

    Attributes:
        type: 片段类型，固定为'link'
        text (str): 原链接
        title (str): 链接标题
        raw_url (str): 解析后的原链接
    """

    type: Literal["link"] = "link"
    text: str = ""
    title: str = ""
    raw_url: str = ""

    @field_validator("raw_url", mode="before")
    @classmethod
    def _coerce_raw_url(cls, v: str | None) -> str:
        return "" if v is None else str(v)


class FragTextModel(BaseModel):
    """纯文本碎片模型。

    Attributes:
        type: 片段类型，固定为'text'
        text (str): 文本内容
    """

    type: Literal["text"] = "text"
    text: str = ""


class FragTiebaPlusModel(BaseModel):
    """贴吧plus广告碎片模型。

    Attributes:
        type: 片段类型，固定为'tieba_plus'
        text (str): 贴吧plus广告描述
        url (str): 解析后的贴吧plus广告跳转链接
    """

    type: Literal["tieba_plus"] = "tieba_plus"
    text: str = ""
    url: str = ""

    @field_validator("url", mode="before")
    @classmethod
    def _coerce_url(cls, v: str | None) -> str:
        return "" if v is None else str(v)


class FragVideoModel(BaseModel):
    """视频碎片模型。

    Attributes:
        type: 片段类型，固定为'video'
        src (str): 视频链接
        cover_src (str): 封面链接
        duration (int): 视频长度（秒）
        width (int): 视频宽度（像素）
        height (int): 视频高度（像素）
        view_num (int): 浏览次数
    """

    type: Literal["video"] = "video"
    src: str = ""
    cover_src: str = ""
    duration: int = 0
    width: int = 0
    height: int = 0
    view_num: int = 0


class FragVoiceModel(BaseModel):
    """音频碎片模型。

    Attributes:
        type: 片段类型，固定为'voice'
        md5 (str): 音频md5
        duration (int): 音频长度（秒）
    """

    type: Literal["voice"] = "voice"
    md5: str = ""
    duration: int = 0


class FragUnknownModel(BaseModel):
    """未知碎片模型。"""

    type: Literal["unknown"] = "unknown"
    raw_data: str = ""


Fragment = (
    FragAtModel
    | FragEmojiModel
    | FragImageModel
    | FragItemModel
    | FragLinkModel
    | FragTextModel
    | FragTiebaPlusModel
    | FragVideoModel
    | FragVoiceModel
    | FragUnknownModel
)


@runtime_checkable
class TypeFragText(Protocol):
    text: str


FRAG_MAP: dict[str, type[Fragment]] = {
    key: cls
    for cls in get_args(Fragment)
    for key in (cls.__name__.removesuffix("Model"), cls.model_fields["type"].default)
}



"""数据模型定义模块。

该模块定义了所有与贴吧数据相关的SQLAlchemy ORM模型和Pydantic验证模型，
包括论坛、用户、主题贴、回复、楼中楼等实体，以及各种内容片段的数据模型。
"""

from __future__ import annotations

from datetime import datetime  # noqa: TC003
from typing import TYPE_CHECKING, Any

from pydantic import TypeAdapter, ValidationError
from sqlalchemy import BIGINT, JSON, DateTime, Index, Integer, String, Text
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.ext.mutable import MutableList
from sqlalchemy.orm import DeclarativeBase, Mapped, foreign, mapped_column, relationship
from sqlalchemy.types import TypeDecorator, TypeEngine

from ..schemas.fragments import FRAG_MAP, Fragment, FragUnknownModel
from ..utils.time_utils import now_with_tz

if TYPE_CHECKING:
    from collections.abc import Callable

    import aiotieba.typing as aiotieba
    from sqlalchemy.engine.interfaces import Dialect

    from .dto import BaseUserDTO, CommentDTO, PostDTO, ThreadDTO

    type AiotiebaType = aiotieba.Thread | aiotieba.Post | aiotieba.Comment


__all__ = [
    "Forum",
    "User",
    "Thread",
    "Post",
    "Comment",
    "Fragment",
]


class Base(DeclarativeBase):
    pass


class FragmentListType(TypeDecorator[list[Fragment]]):
    """自动处理Fragment模型列表的JSON序列化与反序列化。

    自动适配不同数据库的JSON类型。
    """

    impl = JSON
    cache_ok = True

    def __init__(self, fallback: Callable[[], Fragment] | None = None, *args: object, **kwargs: object):
        super().__init__(*args, **kwargs)
        self.adapter: TypeAdapter[Fragment] = TypeAdapter(Fragment)
        self.fallback = fallback

    def load_dialect_impl(self, dialect: Dialect) -> TypeEngine[Any]:
        if dialect.name == "postgresql":
            return dialect.type_descriptor(JSONB())
        return dialect.type_descriptor(JSON())

    def process_bind_param(self, value: list[Fragment] | None, dialect: Dialect) -> list[dict[str, Any]] | None:
        if value is None:
            return None
        return [self.adapter.dump_python(item, mode="json") for item in value]

    def process_result_value(self, value: list[dict[str, Any]] | None, dialect: Dialect) -> list[Fragment] | None:
        if value is None:
            return None
        return [self._validate(item) for item in value]

    def _validate(self, item: dict[str, Any]) -> Fragment:
        if "type" in item:
            if model_cls := FRAG_MAP.get(item["type"]):
                return model_cls.model_construct(**item)

        try:
            return self.adapter.validate_python(item)
        except ValidationError:
            if self.fallback:
                return self.fallback()
            raise


class MixinBase(Base):
    """为SQLAlchemy模型提供通用方法的混入类。"""

    __abstract__ = True

    def to_dict(self) -> dict[str, Any]:
        """将模型实例的列数据转换为字典。

        此方法包含直接映射到数据库表的列，用于批量插入操作。

        Returns:
            dict: 包含模型列名和对应值的字典。
        """
        result = {}
        for c in self.__table__.columns:
            value = getattr(self, c.name)
            result[c.name] = value
        return result


class Forum(MixinBase):
    """贴吧信息数据模型。

    Attributes:
        fid: 论坛ID，主键。
        fname: 论坛名称，建立索引用于快速查询。
        threads: 该论坛下的所有帖子，与Thread模型的反向关系。
    """

    __tablename__ = "forum"

    fid: Mapped[int] = mapped_column(BIGINT, primary_key=True)
    fname: Mapped[str] = mapped_column(String(255), index=True)

    threads: Mapped[list[Thread]] = relationship(
        "Thread",
        back_populates="forum",
        primaryjoin=lambda: Forum.fid == foreign(Thread.fid),
    )


class User(MixinBase):
    """用户数据模型。

    Attributes:
        user_id: 用户user_id，主键。
        portrait: 用户portrait。
        user_name: 用户名。
        nick_name: 用户昵称。
        threads: 该用户发布的所有帖子，与Thread模型的反向关系。
        posts: 该用户发布的所有回复，与Post模型的反向关系。
        comments: 该用户发布的所有评论，与Comment模型的反向关系。
    """

    __tablename__ = "user"

    user_id: Mapped[int] = mapped_column(BIGINT, primary_key=True)
    portrait: Mapped[str] = mapped_column(String(255), nullable=True, index=True)
    user_name: Mapped[str] = mapped_column(String(255), nullable=True, index=True)
    nick_name: Mapped[str] = mapped_column(String(255), nullable=True, index=True)

    threads: Mapped[list[Thread]] = relationship(
        "Thread",
        back_populates="author",
        primaryjoin=lambda: User.user_id == foreign(Thread.author_id),
    )
    posts: Mapped[list[Post]] = relationship(
        "Post",
        back_populates="author",
        primaryjoin=lambda: User.user_id == foreign(Post.author_id),
    )
    comments: Mapped[list[Comment]] = relationship(
        "Comment",
        back_populates="author",
        primaryjoin=lambda: User.user_id == foreign(Comment.author_id),
    )

    @classmethod
    def from_dto(cls, dto: BaseUserDTO) -> User:
        """从UserDTO对象创建User模型实例。

        Args:
            dto: UserDTO对象。

        Returns:
            User: 转换后的User模型实例。
        """
        return cls(
            user_id=dto.user_id,
            portrait=dto.portrait,
            user_name=dto.user_name,
            nick_name=dto.nick_name,
        )


class Thread(MixinBase):
    """主题贴数据模型。

    Attributes:
        tid: 主题贴tid，与create_time组成复合主键。
        create_time: 主题贴创建时间，带时区信息，与tid组成复合主键。
        title: 主题贴标题内容。
        text: 主题贴的纯文本内容。
        contents: 正文内容碎片列表，以JSONB格式存储。
        last_time: 最后回复时间，带时区信息。
        reply_num: 回复数。
        author_level: 作者在主题贴所在吧的等级。
        scrape_time: 数据抓取时间。
        fid: 所属贴吧fid，外键关联到Forum表。
        author_id: 作者user_id，外键关联到User表。
        forum: 所属贴吧对象，与Forum模型的关系。
        author: 作者用户对象，与User模型的关系。
        posts: 该贴子下的所有回复，与Post模型的反向关系。
    """

    __tablename__ = "thread"
    __table_args__ = (
        Index("idx_thread_forum_ctime", "fid", "create_time"),
        Index("idx_thread_forum_ltime", "fid", "last_time"),
        Index("idx_thread_author_time", "author_id", "create_time"),
        Index("idx_thread_author_forum_time", "author_id", "fid", "create_time"),
    )

    tid: Mapped[int] = mapped_column(BIGINT, primary_key=True)
    create_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), primary_key=True)
    title: Mapped[str] = mapped_column(String(255))
    text: Mapped[str] = mapped_column(Text)
    contents: Mapped[list[Fragment] | None] = mapped_column(
        MutableList.as_mutable(FragmentListType(fallback=FragUnknownModel)), nullable=True
    )
    last_time: Mapped[datetime] = mapped_column(DateTime(timezone=True))
    reply_num: Mapped[int] = mapped_column(Integer)
    author_level: Mapped[int] = mapped_column(Integer)
    scrape_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), default=now_with_tz)

    fid: Mapped[int] = mapped_column(BIGINT, index=True)
    author_id: Mapped[int] = mapped_column(BIGINT, index=True)

    forum: Mapped[Forum] = relationship(
        "Forum",
        back_populates="threads",
        primaryjoin=lambda: foreign(Thread.fid) == Forum.fid,
    )
    author: Mapped[User] = relationship(
        "User",
        back_populates="threads",
        primaryjoin=lambda: foreign(Thread.author_id) == User.user_id,
    )
    posts: Mapped[list[Post]] = relationship(
        "Post",
        back_populates="thread",
        primaryjoin=lambda: Thread.tid == foreign(Post.tid),
    )

    @classmethod
    def from_dto(cls, dto: ThreadDTO) -> Thread:
        """从ThreadDTO对象创建Thread模型实例。

        Args:
            dto: ThreadDTO对象。

        Returns:
            Thread: 转换后的Thread模型实例。
        """
        return cls(
            tid=dto.tid,
            create_time=dto.create_time,
            title=dto.title,
            text=dto.text,
            contents=dto.contents,
            last_time=dto.last_time,
            reply_num=dto.reply_num,
            author_level=dto.author.level,
            scrape_time=now_with_tz(),
            fid=dto.fid,
            author_id=dto.author_id,
        )


class Post(MixinBase):
    """回复数据模型。

    Attributes:
        pid: 回复pid，与create_time组成复合主键。
        create_time: 回复创建时间，带时区信息，与pid组成复合主键。
        text: 回复的纯文本内容。
        contents: 回复的正文内容碎片列表，以JSONB格式存储。
        floor: 楼层号。
        reply_num: 该回复下的楼中楼数量。
        author_level: 作者在主题贴所在吧的等级。
        scrape_time: 数据抓取时间。
        tid: 所属贴子tid，外键关联到Thread表。
        author_id: 作者user_id，外键关联到User表。
        thread: 所属主题贴对象，与Thread模型的关系。
        author: 作者用户对象，与User模型的关系。
        comments: 该回复下的所有楼中楼，与Comment模型的反向关系。
    """

    __tablename__ = "post"
    __table_args__ = (
        Index("idx_post_thread_time", "tid", "create_time"),
        Index("idx_post_author_time", "author_id", "create_time"),
    )

    pid: Mapped[int] = mapped_column(BIGINT, primary_key=True)
    create_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), primary_key=True)
    text: Mapped[str] = mapped_column(Text)
    contents: Mapped[list[Fragment] | None] = mapped_column(
        MutableList.as_mutable(FragmentListType(fallback=FragUnknownModel)), nullable=True
    )
    floor: Mapped[int] = mapped_column(Integer)
    reply_num: Mapped[int] = mapped_column(Integer)
    author_level: Mapped[int] = mapped_column(Integer)
    scrape_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), default=now_with_tz)

    tid: Mapped[int] = mapped_column(BIGINT, index=True)
    fid: Mapped[int] = mapped_column(BIGINT, index=True)
    author_id: Mapped[int] = mapped_column(BIGINT, index=True)

    thread: Mapped[Thread] = relationship(
        "Thread",
        back_populates="posts",
        primaryjoin=lambda: foreign(Post.tid) == Thread.tid,
    )
    author: Mapped[User] = relationship(
        "User",
        back_populates="posts",
        primaryjoin=lambda: foreign(Post.author_id) == User.user_id,
    )
    comments: Mapped[list[Comment]] = relationship(
        "Comment",
        back_populates="post",
        primaryjoin=lambda: Post.pid == foreign(Comment.pid),
    )

    @classmethod
    def from_dto(cls, dto: PostDTO) -> Post:
        """从PostDTO对象创建Post模型实例。

        Args:
            dto: PostDTO对象。

        Returns:
            Post: 转换后的Post模型实例。
        """
        return cls(
            pid=dto.pid,
            create_time=dto.create_time,
            text=dto.text,
            contents=dto.contents,
            floor=dto.floor,
            reply_num=dto.reply_num,
            author_level=dto.author.level,
            scrape_time=now_with_tz(),
            tid=dto.tid,
            fid=dto.fid,
            author_id=dto.author_id,
        )


class Comment(MixinBase):
    """楼中楼数据模型。

    Attributes:
        cid: 楼中楼pid，存储为cid以区分，与create_time组成复合主键。
        create_time: 楼中楼创建时间，带时区信息，与cid组成复合主键。
        text: 楼中楼的纯文本内容。
        contents: 楼中楼的正文内容碎片列表，以JSONB格式存储。
        author_level: 作者在主题贴所在吧的等级。
        reply_to_id: 被回复者的user_id，可为空。
        scrape_time: 数据抓取时间。
        pid: 所属回复ID，外键关联到Post表。
        author_id: 作者user_id，外键关联到User表。
        post: 所属回复对象，与Post模型的关系。
        author: 作者用户对象，与User模型的关系。
    """

    __tablename__ = "comment"
    __table_args__ = (
        Index("idx_comment_post_time", "pid", "create_time"),
        Index("idx_comment_author_time", "author_id", "create_time"),
    )

    cid: Mapped[int] = mapped_column(BIGINT, primary_key=True)
    create_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), primary_key=True)
    text: Mapped[str] = mapped_column(Text)
    contents: Mapped[list[Fragment] | None] = mapped_column(
        MutableList.as_mutable(FragmentListType(fallback=FragUnknownModel)), nullable=True
    )
    author_level: Mapped[int] = mapped_column(Integer)
    reply_to_id: Mapped[int | None] = mapped_column(BIGINT, nullable=True)
    scrape_time: Mapped[datetime] = mapped_column(DateTime(timezone=True), default=now_with_tz)

    pid: Mapped[int] = mapped_column(BIGINT, index=True)
    tid: Mapped[int] = mapped_column(BIGINT, index=True)
    fid: Mapped[int] = mapped_column(BIGINT, index=True)
    author_id: Mapped[int] = mapped_column(BIGINT, index=True)

    post: Mapped[Post] = relationship(
        "Post",
        back_populates="comments",
        primaryjoin=lambda: foreign(Comment.pid) == Post.pid,
    )
    author: Mapped[User] = relationship(
        "User",
        back_populates="comments",
        primaryjoin=lambda: foreign(Comment.author_id) == User.user_id,
    )

    @classmethod
    def from_dto(cls, dto: CommentDTO) -> Comment:
        """从CommentDTO对象创建Comment模型实例。

        Args:
            dto: CommentDTO对象。

        Returns:
            Comment: 转换后的Comment模型实例。
        """
        return cls(
            cid=dto.cid,
            create_time=dto.create_time,
            text=dto.text,
            contents=dto.contents,
            author_level=dto.author.level,
            reply_to_id=dto.reply_to_id,
            scrape_time=now_with_tz(),
            pid=dto.pid,
            tid=dto.tid,
            fid=dto.fid,
            author_id=dto.author_id,
        )



from datetime import datetime

from tiebameow.models.dto import (
    BaseUserDTO,
    ShareThreadDTO,
    ThreadDTO,
    ThreadUserDTO,
)
from tiebameow.schemas.fragments import FragTextModel


def test_base_user_dto() -> None:
    user = BaseUserDTO(user_id=1, portrait="portrait", user_name="user_name", nick_name_new="nick_name")
    assert user.nick_name == "nick_name"
    assert user.show_name == "nick_name"

    user_no_nick = BaseUserDTO(user_id=1, portrait="portrait", user_name="user_name", nick_name_new="")
    assert user_no_nick.nick_name == ""
    assert user_no_nick.show_name == "user_name"


def test_thread_user_dto() -> None:
    user = ThreadUserDTO(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name_new="nick_name",
        level=1,
        glevel=1,
        gender="MALE",
        icons=[],
        is_bawu=False,
        is_vip=False,
        is_god=False,
        priv_like="PUBLIC",
        priv_reply="ALL",
    )
    assert user.level == 1
    assert user.gender == "MALE"


def test_thread_dto() -> None:
    author = ThreadUserDTO(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name_new="nick_name",
        level=1,
        glevel=1,
        gender="MALE",
        icons=[],
        is_bawu=False,
        is_vip=False,
        is_god=False,
        priv_like="PUBLIC",
        priv_reply="ALL",
    )
    share_origin = ShareThreadDTO(pid=0, tid=0, fid=0, fname="", author_id=0, title="", contents=[])
    thread = ThreadDTO(
        pid=1,
        tid=1,
        fid=1,
        fname="fname",
        author_id=1,
        author=author,
        title="title",
        contents=[FragTextModel(text="content")],
        is_good=False,
        is_top=False,
        is_share=False,
        is_hide=False,
        is_livepost=False,
        is_help=False,
        agree_num=0,
        disagree_num=0,
        reply_num=0,
        view_num=0,
        share_num=0,
        create_time=datetime.now(),
        last_time=datetime.now(),
        thread_type=0,
        tab_id=0,
        share_origin=share_origin,
    )
    assert thread.title == "title"
    assert len(thread.contents) == 1
    assert isinstance(thread.contents[0], FragTextModel)
    assert thread.contents[0].text == "content"
